@@ -9,7 +9,6 @@ from gi.repository import Gio
 
 try:
     import sys
-    from gi.repository import Pango
     import os
     import commands
     import gettext
@@ -18,23 +17,21 @@ try:
     import gc
     import ctypes
     from ctypes import *
-    import capi
     import xdg.Config
     import keybinding
+    import pointerMonitor
 except Exception, e:
     print e
     sys.exit( 1 )
 
 GObject.threads_init()
 
-gtk = CDLL("libgtk-x11-2.0.so.0")
 gdk = CDLL("libgdk-x11-2.0.so.0")
 
 # Rename the process
 architecture = commands.getoutput("uname -a")
 if (architecture.find("x86_64") >= 0):
-    import ctypes
-    libc = ctypes.CDLL('libc.so.6')
+    libc = CDLL('libc.so.6')
     libc.prctl(15, 'mintmenu', 0, 0, 0)
 else:
     import dl
@@ -87,43 +84,39 @@ class MainWindow( object ):
         self.panesToColor = [ ]
         self.headingsToColor = [ ]
 
-        self.window.connect( "map-event", self.onMap )
-        self.window.connect( "show", self.onShow )
-        self.window.connect( "unmap-event", self.onUnmap )
-        self.window.connect( "button-press-event", self.onButtonPress )
         self.window.connect( "key-press-event", self.onKeyPress )
-        self.window.connect( "grab-broken-event", self.onGrabBroken )
+        self.window.connect( "focus-in-event", self.onFocusIn )
+        self.loseFocusId = self.window.connect( "focus-out-event", self.onFocusOut )
+        self.loseFocusBlocked = False
 
         self.window.stick()
 
         plugindir = os.path.join( os.path.expanduser( "~" ), ".linuxmint/mintMenu/plugins" )
         sys.path.append( plugindir )
-                      
-        self.getSetGSettingEntries()
-        self.SetupMintMenuBorder()
-        self.SetupMintMenuOpacity()
 
-        self.tooltips = Gtk.Tooltips()
-        if self.globalEnableTooltips and self.enableTooltips:
-            self.tooltips.enable()
-        else:
-            self.tooltips.disable()
-
-        self.PopulatePlugins();
+        self.panelSettings = Gio.Settings.new("org.mate.panel")
+        self.panelSettings.connect( "changed::tooltips-enabled", self.toggleTooltipsEnabled )
 
         self.settings.connect( "changed::plugins-list", self.RegenPlugins )
-                
         self.settings.connect( "changed::start-with-favorites", self.toggleStartWithFavorites )
-        globalsettings = Gio.Settings.new("org.mate.panel")
-        globalsettings.connect( "changed::tooltips-enabled", self.toggleTooltipsEnabled )
         self.settings.connect( "changed::tooltips-enabled", self.toggleTooltipsEnabled )
-
         self.settings.connect( "changed::use-custom-color", self.toggleUseCustomColor )
         self.settings.connect( "changed::custom-border-color", self.toggleCustomBorderColor )
         self.settings.connect( "changed::custom-heading-color", self.toggleCustomHeadingColor )
         self.settings.connect( "changed::custom-color", self.toggleCustomBackgroundColor )
         self.settings.connect( "changed::border-width", self.toggleBorderWidth )
         self.settings.connect( "changed::opacity", self.toggleOpacity )
+        
+        self.getSetGSettingEntries()
+
+        self.tooltips = Gtk.Tooltips()
+        if self.globalEnableTooltips and self.enableTooltips:
+            self.tooltips.enable()
+        else:
+            self.tooltips.disable()
+            
+        self.PopulatePlugins();
+        self.firstTime = True;
 
     def on_window1_destroy (self, widget, data=None):
         Gtk.main_quit()
@@ -159,9 +152,7 @@ class MainWindow( object ):
 
     def toggleUseCustomColor( self, settings, key, args = None ):
         self.usecustomcolor = settings.get_boolean(key)
-        self.SetupMintMenuBorder()
-        self.SetPaneColors( self.panesToColor )
-        self.SetHeadingStyle( self.headingsToColor )
+        self.loadTheme()
 
     def toggleCustomBorderColor( self, settings, key, args = None ):
         self.custombordercolor = settings.get_string(key)
@@ -189,14 +180,13 @@ class MainWindow( object ):
         self.enableTooltips       = self.settings.get_boolean( "tooltips-enabled" )        
         self.startWithFavorites   = self.settings.get_boolean( "start-with-favorites" )
         
-        mate_settings = Gio.Settings.new("org.mate.panel")
-        self.globalEnableTooltips = mate_settings.get_boolean( "tooltips-enabled" )
+        self.globalEnableTooltips = self.panelSettings.get_boolean( "tooltips-enabled" )
 
-    def SetupMintMenuBorder( self ):
+    def SetupMintMenuBorder( self, defaultStyle = None ):
         if self.usecustomcolor:
             self.window.modify_bg( Gtk.StateType.NORMAL, Gdk.color_parse( self.custombordercolor ) )
-        # else:
-        #     self.window.modify_bg( Gtk.StateType.NORMAL, self.window.rc_get_style().bg[ Gtk.StateType.SELECTED ] )
+        elif defaultStyle is not None:
+            self.window.modify_bg( Gtk.StateType.NORMAL, defaultStyle.lookup_color('bg_color')[1] )
         self.border.set_padding( self.borderwidth, self.borderwidth, self.borderwidth, self.borderwidth )        
 
     def SetupMintMenuOpacity( self ):
@@ -227,10 +217,9 @@ class MainWindow( object ):
         PluginPane.show()
         PaneLadder = Gtk.VBox( False, 0 )
         PluginPane.add( PaneLadder )
-        self.SetPaneColors( [ PluginPane ] )
         ImageBox = Gtk.EventBox()
-        self.SetPaneColors( [ ImageBox ] )
         ImageBox.show()
+        self.panesToColor.extend( [ PluginPane, ImageBox ] )
 
         seperatorImage = GdkPixbuf.Pixbuf.new_from_file( self.dottedfile )
 
@@ -284,9 +273,7 @@ class MainWindow( object ):
                     print u"Unable to load " + plugin + " plugin :-("
 
 
-                self.SetPaneColors( [MyPlugin.content_holder] )
-
-
+                self.panesToColor.append( MyPlugin.content_holder )
                 MyPlugin.content_holder.show()
 
                 VBox1 = Gtk.VBox( False, 0 )
@@ -295,14 +282,14 @@ class MainWindow( object ):
                     Align1 = Gtk.Alignment.new( 0, 0, 0, 0 )
                     Align1.set_padding( 10, 5, 10, 0 )
                     Align1.add( Label1 )
-                    self.SetHeadingStyle( [Label1] )
+                    self.headingsToColor.append( Label1 )
                     Align1.show()
                     Label1.show()
 
                     if not hasattr( MyPlugin, 'sticky' ) or MyPlugin.sticky == True:
                         heading = Gtk.EventBox()
                         Align1.set_padding( 0, 0, 10, 0 )
-                        self.SetPaneColors( [heading] )
+                        heading.set_visible_window( False )
                         heading.set_size_request( MyPlugin.width, 30 )
                     else:
                         heading = Gtk.HBox()
@@ -331,7 +318,9 @@ class MainWindow( object ):
                     if hasattr( MyPlugin, 'height' ):
                         MyPlugin.content_holder.set_size_request( -1, MyPlugin.height )
                     if hasattr( MyPlugin, 'itemstocolor' ):
-                        self.SetPaneColors( MyPlugin.itemstocolor )                   
+                        self.panesToColor.extend( MyPlugin.itemstocolor )
+                    if hasattr( MyPlugin, 'headingstocolor' ):
+                        self.headingsToColor.extend( MyPlugin.headingstocolor )
                 except:
                     # create traceback
                     info = sys.exc_info()
@@ -349,9 +338,8 @@ class MainWindow( object ):
                 PluginPane = Gtk.EventBox()
                 PaneLadder = Gtk.VBox( False, 0 )
                 PluginPane.add( PaneLadder )
-                self.SetPaneColors( [PluginPane] )
                 ImageBox = Gtk.EventBox()
-                self.SetPaneColors( [ImageBox] )
+                self.panesToColor.extend( [ PluginPane, ImageBox ] )
                 ImageBox.show()
                 PluginPane.show_all()
 
@@ -373,22 +361,30 @@ class MainWindow( object ):
         self.tooltips.disable()
         #print u"Loading", (time.time() - start), "s"
 
-    def SetPaneColors( self, items ):
-        for item in items:
-            if item not in self.panesToColor:
-                self.panesToColor.append( item )
+    # A little hacky but works
+    def getDefaultStyle( self ):
+        widget = Gtk.EventBox()
+        widget.show()
+        return Gtk.rc_get_style(widget)
+        
+    def loadTheme( self ):
+        defaultStyle = self.getDefaultStyle()
+        self.SetPaneColors( self.panesToColor, defaultStyle )
+        self.SetupMintMenuBorder( defaultStyle )
+        self.SetHeadingStyle( self.headingsToColor )
+        
+    def SetPaneColors( self, items, defaultStyle = None ):
         if self.usecustomcolor:
             for item in items:
                 item.modify_bg( Gtk.StateType.NORMAL, Gdk.color_parse( self.customcolor ) )
-        # else:
-        #     for item in items:
-        #         item.modify_bg( Gtk.StateType.NORMAL, self.paneholder.rc_get_style().bg[ Gtk.StateType.NORMAL ] )
+                # TODO: Changing background color isn't working for pixmaps! The following does not work:
+                item.get_style().bg_pixmap[Gtk.StateType.NORMAL] = None
+        elif defaultStyle is not None:
+            for item in items:
+                item.modify_bg( Gtk.StateType.NORMAL, defaultStyle.lookup_color('bg_color')[1] )
+                item.get_style().bg_pixmap[Gtk.StateType.NORMAL] = defaultStyle.bg_pixmap[Gtk.StateType.NORMAL]
 
     def SetHeadingStyle( self, items ):
-        for item in items:
-            if item not in self.headingsToColor:
-                self.headingsToColor.append( item )
-
         if self.usecustomcolor:
             color = self.customheadingcolor
         else:
@@ -430,84 +426,58 @@ class MainWindow( object ):
 
         self.getSetGSettingEntries()
         self.PopulatePlugins()
+        self.loadTheme()
 
         #print NAME+u" reloaded"
 
+    def onKeyPress( self, widget, event ):
+        if event.keyval == Gdk.KEY_Escape:
+            self.hide()
+            return True
+        return False
 
     def show( self ):
         self.window.present()
+        
+        # Hack for opacity not showing on first composited draw
+        if self.firstTime:
+            self.firstTime = False
+            self.SetupMintMenuOpacity()
+
+        self.window.window.focus( Gdk.CURRENT_TIME )
+            
+        for plugin in self.plugins.values():
+            if hasattr( plugin, "onShowMenu" ):
+                plugin.onShowMenu()
 
         if ( "applications" in self.plugins ) and ( hasattr( self.plugins["applications"], "focusSearchEntry" ) ):
             if (self.startWithFavorites):
                 self.plugins["applications"].changeTab(0)
             self.plugins["applications"].focusSearchEntry()
 
-    def grab( self ):
-        gdk.gdk_pointer_grab (hash(self.window.window), True, Gdk.EventMask.BUTTON_PRESS_MASK, None, None, Gdk.CURRENT_TIME)
-        Gdk.keyboard_grab( self.window.window, False, Gdk.CURRENT_TIME )
-        Gtk.grab_add(self.window)
-
-    def ungrab( self ):
-        Gtk.grab_remove(self.window)
-        self.window.hide()
-        Gdk.pointer_ungrab(Gdk.CURRENT_TIME)
-        Gdk.keyboard_ungrab(Gdk.CURRENT_TIME)
-
-    def onMap( self, widget, event ):
-        self.grab()
-
-    def onShow( self, widget ):
-        for plugin in self.plugins.values():
-            if hasattr( plugin, "onShowMenu" ):
-                plugin.onShowMenu()
-
-    def onUnmap( self, widget, event ):
-        self.ungrab()
-
+    def hide( self, forceHide = False ):
         for plugin in self.plugins.values():
             if hasattr( plugin, "onHideMenu" ):
                 plugin.onHideMenu()
-
-    def onKeyPress( self, widget, event ):
-        if event.keyval == Gdk.KEY_Escape or self.keybinder.is_hotkey(event.keyval, event.get_state()):
-            self.hide()
-            return True
-        return False
-
-    def onButtonPress( self, widget, event ):
-        # Check if the pointer is within the menu, else hide the menu
-        winatptr = Gdk.window_at_pointer()
-        if winatptr:
-            win = winatptr[0]
-            while win:
-                if win == self.window.window:
-                    break
-                win = capi.get_widget(gdk.gdk_window_get_parent (hash(win)))
-            if not win:
-                self.hide( True )
-        else:
-            self.hide( True )
-
-        return True
-
-    def onGrabBroken( self, widget, event ):
-        if event.grab_broken.grab_window:
-            try:
-                win = event.grab_broken.grab_window
-                data = c_void_p()
-                gdk.gdk_window_get_user_data(hash(win), byref(data))
-                theft = capi.get_widget(ctypes.cast(data, POINTER(capi._PyGObject_Functions)))
-                theft.connect( "event", self.onGrabTheftEvent )
-            except Exception, detail:
-                print detail
-                self.window.hide()
-
-    def onGrabTheftEvent( self, widget, event ):
-        if event.type == Gdk.EventType.UNMAP or event.type == Gdk.EventType.SELECTION_CLEAR:
-            self.grab()
-
-    def hide(self, forceHide = False):        
+                
         self.window.hide()
+        
+    def onFocusIn( self, *args ):
+        if self.loseFocusBlocked:
+            self.window.handler_unblock( self.loseFocusId )
+            self.loseFocusBlocked = False
+
+        return False  
+        
+    def onFocusOut( self, *args):            
+        if self.window.get_visible():
+            self.hide()
+        return False
+        
+    def stopHiding( self ):
+        if not self.loseFocusBlocked:
+            self.window.handler_block( self.loseFocusId )
+            self.loseFocusBlocked = True
 
 class MenuWin( object ):
     def __init__( self, applet, iid ):
@@ -523,8 +493,8 @@ class MenuWin( object ):
         self.settings.connect( "changed::hot-key", self.hotkeyChanged )
         self.loadSettings()
 
-        mate_settings = Gio.Settings.new("org.mate.interface")
-        mate_settings.connect( "changed::gtk-theme", self.changeTheme )
+        self.mate_settings = Gio.Settings.new("org.mate.interface")
+        self.mate_settings.connect( "changed::gtk-theme", self.changeTheme )
 
         self.createPanelButton()
 
@@ -534,33 +504,69 @@ class MenuWin( object ):
         self.applet.connect("enter-notify-event", self.enter_notify)
         self.applet.connect("leave-notify-event", self.leave_notify)
         self.mainwin = MainWindow( self.button_box, self.settings, self.keybinder )
-        self.mainwin.window.connect( "map-event", lambda *args: self.applet.set_state( Gtk.StateType.SELECTED ) )
-        self.mainwin.window.connect( "unmap-event", lambda *args: self.applet.set_state( Gtk.StateType.NORMAL ) )
+        self.mainwin.window.connect( "map-event", self.onWindowMap )
+        self.mainwin.window.connect( "unmap-event", self.onWindowUnmap )
+        self.mainwin.window.connect( "enter-notify-event", self.onWindowEnter )
+        self.mainwin.window.connect( "leave-notify-event", self.onWindowLeave )
+        self.mainwin.window.connect( "realize", self.onRealize )
         self.mainwin.window.connect( "size-allocate", lambda *args: self.positionMenu() )
 
         self.mainwin.window.set_name("mintmenu") # Name used in Gtk RC files
+        self.applyTheme()
+        self.mainwin.loadTheme()
 
         if self.mainwin.icon:
             Gtk.Window.set_default_icon_name( self.mainwin.icon )
 
         self.bind_hot_key()
+        self.applet.set_can_focus(False)
+
+        self.pointerMonitor = pointerMonitor.PointerMonitor()
+        self.pointerMonitor.connect("activate", self.onPointerOutside)
+        
+    def onWindowMap( self, *args ):
+        self.applet.set_state( Gtk.StateType.SELECTED )
+        self.keybinder.set_focus_window( self.mainwin.window.window )
+        self.pointerMonitor.grabPointer()
+        return False
+        
+    def onWindowUnmap( self, *args ):
+        self.applet.set_state( Gtk.StateType.NORMAL )
+        self.keybinder.set_focus_window()
+        self.pointerMonitor.ungrabPointer()
+        return False
+        
+    def onWindowEnter(self, applet, event):
+        self.pointerMonitor.ungrabPointer()
+        return False
+
+    def onWindowLeave(self, applet, event):
+        self.pointerMonitor.grabPointer()
+        return False
+        
+    def onRealize( self, *args):
+        self.pointerMonitor.addWindowToMonitor( self.mainwin.window.window )
+        self.pointerMonitor.addWindowToMonitor( self.applet.window )
+        self.pointerMonitor.start()
+        return False
+     
+    def onPointerOutside(self, *args):
+        self.mainwin.hide()
+        return True
 
     def onBindingPress(self, binder):
-        try:
-            if self.mainwin.window.get_visible():
-                self.mainwin.window.hide()
-                self.mainwin.toggle.set_active(False)
-            else:
-                MenuWin.showMenu(self,self.mainwin.toggle)
-                self.mainwin.window.show()
-                #self.mainwin.wTree.get_widget( 'PluginTabs' ).set_curremenu_editor = SetGconf( self.client, "string", "/apps/usp/menu_editor", "mozo" )
-        except Exception, cause:
-            print cause
+        self.toggleMenu()
+        return True
 
     def enter_notify(self, applet, event):
         self.do_image(self.buttonIcon, True)
 
     def leave_notify(self, applet, event):
+		# Hack for mate-panel-test-applets focus issue (this can be commented)
+        if event.state & Gdk.ModifierType.BUTTON1_MASK and applet.state & Gtk.StateType.SELECTED:
+            if event.x >= 0 and event.y >= 0 and event.x < applet.window.get_width() and event.y < applet.window.get_height():
+                self.mainwin.stopHiding()
+                
         self.do_image(self.buttonIcon, False)
 
     def do_image(self, image_file, saturate):
@@ -584,20 +590,19 @@ class MenuWin( object ):
             self.button_box = Gtk.HBox()
             self.button_box.pack_start( self.button_icon, False, False, 0 )
             self.button_box.pack_start( self.systemlabel, False, False, 0 )
-
             self.button_icon.set_padding( 5, 0 )
         # if we have a vertical panel
         elif self.applet.get_orient() == MatePanelApplet.AppletOrient.LEFT:
             self.button_box = Gtk.VBox()
             self.systemlabel.set_angle( 270 )
-            self.button_box.pack_start( self.systemlabel , True, True, 0)
-            self.button_box.pack_start( self.button_icon , True, True, 0)
-            self.button_icon.set_padding( 5, 0 )
+            self.button_box.pack_start( self.button_icon , False, False, 0)
+            self.button_box.pack_start( self.systemlabel , False, False, 0)
+            self.button_icon.set_padding( 0, 5 )
         elif self.applet.get_orient() == MatePanelApplet.AppletOrient.RIGHT:
             self.button_box = Gtk.VBox()
             self.systemlabel.set_angle( 90 )
-            self.button_box.pack_start( self.button_icon , True, True, 0)
-            self.button_box.pack_start( self.systemlabel , True, True, 0)
+            self.button_box.pack_start( self.systemlabel , False, False, 0)
+            self.button_box.pack_start( self.button_icon , False, False, 0)
             self.button_icon.set_padding( 0, 5 )
 
         self.button_box.set_homogeneous( False )
@@ -619,12 +624,11 @@ class MenuWin( object ):
     def changeTheme(self, *args):        
         self.reloadSettings()
         self.applyTheme()
-        self.mainwin.RegenPlugins()
+        self.mainwin.loadTheme()
     
     def applyTheme(self):
         style_settings = Gtk.Settings.get_default()
-        mate_settings = Gio.Settings.new("org.mate.interface")
-        desktop_theme = mate_settings.get_string('gtk-theme')
+        desktop_theme = self.mate_settings.get_string('gtk-theme')
         if self.theme_name == "default":
             style_settings.set_property("gtk-theme-name", desktop_theme)        
         else:
@@ -643,12 +647,12 @@ class MenuWin( object ):
         elif self.applet.get_orient() == MatePanelApplet.AppletOrient.LEFT:
             tmpbox = Gtk.VBox()
             self.systemlabel.set_angle( 270 )
-            self.button_box.reorder_child( self.button_icon, 1 )
+            self.button_box.reorder_child( self.button_icon, 0 )
             self.button_icon.set_padding( 0, 5 )
         elif self.applet.get_orient() == MatePanelApplet.AppletOrient.RIGHT:
             tmpbox = Gtk.VBox()
             self.systemlabel.set_angle( 90 )
-            self.button_box.reorder_child( self.button_icon, 0 )
+            self.button_box.reorder_child( self.button_icon, 1 )
             self.button_icon.set_padding( 0, 5 )
 
         tmpbox.set_homogeneous( False )
@@ -688,10 +692,8 @@ class MenuWin( object ):
             pass
 
     def hotkeyChanged (self, schema, key):
-        self.keybinder.ungrab()
         self.hotkeyText =  self.settings.get_string( "hot-key" )
-        if self.hotkeyText != "":
-            self.keybinder.grab(self.hotkeyText)
+        self.keybinder.rebind(self.hotkeyText)
 
     def sizeButton( self ):
         if self.hideIcon:
@@ -706,8 +708,7 @@ class MenuWin( object ):
         self.systemlabel.size_request(sl_req)
         if self.applet.get_orient() == MatePanelApplet.AppletOrient.UP or self.applet.get_orient() == MatePanelApplet.AppletOrient.DOWN:
            if self.hideIcon:
-               self.systemlabel.size_request(sl_req)
-               self.applet.set_size_request( sl_req.width + 2, -1 )
+               self.applet.set_size_request( sl_req.width + 2, bi_req.height )
            else:
                self.applet.set_size_request( sl_req.width + bi_req.width + 5, bi_req.height )
         else:
@@ -737,7 +738,7 @@ class MenuWin( object ):
             about.set_license(gpl)
         except Exception, detail:
             print detail
-        about.set_comments( _("Advanced Gnome Menu") )
+        about.set_comments( _("Advanced MATE Menu") )
       #  about.set_authors( ["Clement Lefebvre <clem@linuxmint.com>", "Lars-Peter Clausen <lars@laprican.de>"] )
         about.set_translator_credits(("translator-credits") )
         #about.set_copyright( _("Based on USP from S.Chanderbally") )
