@@ -6,8 +6,6 @@ gi.require_version("Gtk", "2.0")
 from gi.repository import Gtk, Pango, Gdk, Gio, GLib
 
 import os
-#import mateconf
-import fnmatch
 import time
 import string
 import gettext
@@ -16,7 +14,6 @@ import commands
 import subprocess
 import filecmp
 import ctypes
-import capi
 from ctypes import *
 from easybuttons import *
 from execute import Execute
@@ -253,7 +250,7 @@ class pluginclass( object ):
         self.builder.get_object("label7").set_text(_("All applications"))
         self.builder.get_object("label2").set_text(_("Applications"))                
         
-        self.mintMenuWin.SetHeadingStyle( [self.builder.get_object("label6"),self.builder.get_object("label2")] )
+        self.headingstocolor = [self.builder.get_object("label6"),self.builder.get_object("label2")]
 
         self.numApps = 0
         # These properties are NECESSARY to maintain consistency
@@ -268,7 +265,7 @@ class pluginclass( object ):
         self.content_holder =self.builder.get_object( "Applications" )
 
         # Items to get custom colors
-        self.itemstocolor = [self.builder.get_object( "viewport1" ),self.builder.get_object( "viewport2" ),self.builder.get_object( "viewport3" ),self.builder.get_object( "notebook2" ) ]
+        self.itemstocolor = [self.builder.get_object( "viewport1" ),self.builder.get_object( "viewport2" ),self.builder.get_object( "viewport3" ) ]
 
         # Unset all timers
         self.filterTimer = None
@@ -341,7 +338,7 @@ class pluginclass( object ):
         self.panel = "top"
         self.panel_position = -1
 
-        self.builder.get_object("searchButton").connect( "button-release-event", self.searchPopup )        
+        self.builder.get_object("searchButton").connect( "button-press-event", self.searchPopup )
 
     def refresh_apt_cache(self):
         if self.useAPT:
@@ -564,30 +561,41 @@ class pluginclass( object ):
     def onHideMenu( self ):
         self.settings.set( "int", "last-active-tab", self.lastActiveTab )
 
-    def changeTab( self, tabNum ):
+    def changeTab( self, tabNum, clear = True ):
         notebook = self.builder.get_object( "notebook2" )
         if tabNum == 0:
             notebook.set_current_page( 0 )
         elif tabNum == 1:
             notebook.set_current_page( 1 )
 
+        self.focusSearchEntry(clear)
         self.lastActiveTab = tabNum
-        self.focusSearchEntry()
+
 
     def Todos( self ):
-
+        self.searchEntry.connect( "popup-menu", self.blockOnPopup )
+        self.searchEntry.connect( "button-press-event", self.blockOnRightPress )
         self.searchEntry.connect( "changed", self.Filter )
         self.searchEntry.connect( "activate", self.Search )
         self.showAllAppsButton.connect( "clicked", lambda widget: self.changeTab( 1 ) )
         self.showFavoritesButton.connect( "clicked", lambda widget: self.changeTab( 0 ) )
         self.buildButtonList()
+        
+    def blockOnPopup( self, *args ):
+        self.mintMenuWin.stopHiding()
+        return False
+        
+    def blockOnRightPress( self, widget, event ):
+        if event.button == 3:
+            self.mintMenuWin.stopHiding()
+        return False
 
-    def focusSearchEntry( self ):
+    def focusSearchEntry( self, clear = True ):
         # grab_focus() does select all text,
         # restoring the original selection is somehow broken, so just select the end
         # of the existing text, that's the most likely candidate anyhow
         self.searchEntry.grab_focus()
-        if self.rememberFilter:
+        if self.rememberFilter or not clear:
             gtk.gtk_editable_set_position(hash(self.searchEntry), -1)
         else:
             self.searchEntry.set_text("")
@@ -640,7 +648,6 @@ class pluginclass( object ):
         separator.set_visible_window(False)
         separator.set_size_request(-1, 20)       
         separator.type = "separator"        
-        self.mintMenuWin.SetPaneColors( [ separator ] )
         separator.show_all()
         self.applicationsBox.add(separator)
         self.suggestions.append(separator)        
@@ -718,8 +725,7 @@ class pluginclass( object ):
                 last_separator.add(Gtk.HSeparator())
                 last_separator.set_visible_window(False)
                 last_separator.set_size_request(-1, 20)       
-                last_separator.type = "separator"   
-                self.mintMenuWin.SetPaneColors( [  last_separator ] )     
+                last_separator.type = "separator"       
                 last_separator.show_all()
                 self.applicationsBox.add(last_separator)
                 self.suggestions.append(last_separator)
@@ -774,8 +780,7 @@ class pluginclass( object ):
                     last_separator.add(Gtk.HSeparator())
                     last_separator.set_visible_window(False)
                     last_separator.set_size_request(-1, 20)       
-                    last_separator.type = "separator"   
-                    self.mintMenuWin.SetPaneColors( [  last_separator ] )     
+                    last_separator.type = "separator"        
                     last_separator.show_all()
                     self.applicationsBox.add(last_separator)
                     self.suggestions.append(last_separator)
@@ -813,8 +818,7 @@ class pluginclass( object ):
             else:
                 text = widget.get_text()
                 if self.lastActiveTab != 1:
-                    self.changeTab( 1 )
-                    widget.set_text( text ) 
+                    self.changeTab( 1, clear = False )
                 text = widget.get_text()
                 showns = False # Are any app shown?
                 shownList = []
@@ -878,15 +882,18 @@ class pluginclass( object ):
                 i.released()
                 i.set_relief( Gtk.ReliefStyle.NONE )
             widget.set_relief( Gtk.ReliefStyle.HALF )
-            widget.grab_focus()
-
-            self.searchEntry.set_text( "" )
    
         self.applicationsScrolledWindow.get_vadjustment().set_value( 0 )
+
+    def FilterAndClear( self, widget, category = None ):
+        self.searchEntry.set_text( "" )
+        self.Filter( widget, category )
         
     # Forward all text to the search box
     def keyPress( self, widget, event ):
         if event.string.strip() != "" or event.keyval == Gdk.KEY_BackSpace:
+            self.searchEntry.grab_focus()
+            gtk.gtk_editable_set_position(hash(self.searchEntry), -1)
             self.searchEntry.event( event )
             return True
 
@@ -950,10 +957,8 @@ class pluginclass( object ):
                 mTree.append(propsMenuItem)
 
                 mTree.show_all()
+                self.mintMenuWin.stopHiding()
                 gtk.gtk_menu_popup(hash(mTree), None, None, None, None, ev.button, ev.time)
-                #self.mintMenuWin.grab()
-                mTree.connect( 'deactivate', self.onMenuPopupDeactivate)
-
             else:
                 mTree = Gtk.Menu()
                 mTree.set_events(Gdk.EventMask.POINTER_MOTION_MASK | Gdk.EventMask.POINTER_MOTION_HINT_MASK |
@@ -971,9 +976,8 @@ class pluginclass( object ):
                 removeMenuItem.connect( "activate", self.onFavoritesRemove, widget )
                 insertSpaceMenuItem.connect( "activate", self.onFavoritesInsertSpace, widget, insertBefore )
                 insertSeparatorMenuItem.connect( "activate", self.onFavoritesInsertSeparator, widget, insertBefore )
+                self.mintMenuWin.stopHiding()
                 gtk.gtk_menu_popup(hash(mTree), None, None, None, None, ev.button, ev.time)
-                #self.mintMenuWin.grab()
-                mTree.connect( 'deactivate', self.onMenuPopupDeactivate)
 
     def menuPopup( self, widget, event ):
         if event.button == 3:
@@ -1034,11 +1038,9 @@ class pluginclass( object ):
                 startupMenuItem.set_active( False )
                 startupMenuItem.connect( "toggled", self.onAddToStartup, widget )
 
-            mTree.connect( 'deactivate', self.onMenuPopupDeactivate)
-            gtk.gtk_menu_popup(hash(mTree), None, None, None, None, 0, 0)
+            self.mintMenuWin.stopHiding()
+            gtk.gtk_menu_popup(hash(mTree), None, None, None, None, event.button, event.time)
 
-    def onMenuPopupDeactivate( self, widget):
-        self.mintMenuWin.grab()
     
     def searchPopup( self, widget=None, event=None ):    
         menu = Gtk.Menu()
@@ -1114,14 +1116,14 @@ class pluginclass( object ):
         
         menu.show_all()
 
-        gtk.gtk_menu_popup(hash(menu), None, None, None, None, 3, 0)
+        self.mintMenuWin.stopHiding()
+        gtk.gtk_menu_popup(hash(menu), None, None, None, None, event.button, event.time)
 
         #menu.attach_to_widget(self.searchButton, None)
         #menu.reposition()
         #menu.reposition()
         #self.mintMenuWin.grab()
-        menu.connect( 'deactivate', self.onMenuPopupDeactivate)
-        self.focusSearchEntry()
+        self.focusSearchEntry(clear = False)
         return True
         
     def pos_func(self, menu=None):
@@ -1348,10 +1350,9 @@ class pluginclass( object ):
         space = Gtk.EventBox()
         space.set_size_request( -1, 20 )
         space.set_visible_window(False)
-        space.connect( "button_release_event", self.favPopup )
+        space.connect( "button-press-event", self.favPopup )
         space.type = "space"
 
-        self.mintMenuWin.SetPaneColors( [ space ] )
         space.show()
 
         return space
@@ -1361,13 +1362,12 @@ class pluginclass( object ):
         separator.set_size_request( -1, 20 )
         separator.type = "separator"
 
-        self.mintMenuWin.SetPaneColors( [ separator ] )
         separator.show_all()
         box = Gtk.EventBox()
         box.type = "separator"
         box.add(separator)
         box.set_visible_window(False)
-        box.connect( "button_release_event", self.favPopup )
+        box.connect( "button-press-event", self.favPopup )
         box.show_all()
         return box
 
@@ -1405,7 +1405,7 @@ class pluginclass( object ):
             if favButton.appExec:
                 favButton.show()
                 favButton.connect( "popup-menu", self.favPopup )
-                favButton.connect( "button_release_event", self.favPopup )
+                favButton.connect( "button-press-event", self.favPopup )
                 favButton.connect( "focus-in-event", self.scrollItemIntoView )
                 favButton.connect( "clicked", lambda w: self.mintMenuWin.hide() )
 
@@ -1677,7 +1677,7 @@ class pluginclass( object ):
                     else:
                         item["button"].mouseOverHandlerIds = None
 
-                    item["button"].connect( "clicked", self.Filter, item["filter"] )
+                    item["button"].connect( "clicked", self.FilterAndClear, item["filter"] )
                     item["button"].connect( "focus-in-event", self.categoryBtnFocus, item["filter"] )
                     item["button"].show()
 
