@@ -278,12 +278,11 @@ class ApplicationLauncher( easyButton ):
         icon = self.getIcon( Gtk.IconSize.DND )
         if icon:
             iconName, s = icon.get_icon_name()
-            c = c_char_p(iconName.encode('ascii', 'ignore'))
+            c = c_char_p(iconName.decode('utf-8', 'ignore').encode('ascii', 'ignore'))
             gtk.gtk_drag_source_set_icon_name( hash(self), c)
 
         self.connectSelf( "focus-in-event", self.onFocusIn )
         self.connectSelf( "focus-out-event", self.onFocusOut )
-        self.connectSelf( "enter-notify-event", self.onEnterNotify )
         self.connectSelf( "clicked", self.execute )
 
 
@@ -296,11 +295,11 @@ class ApplicationLauncher( easyButton ):
             self.appExec = self.strip_accents(desktopItem.getExec())
             self.appIconName = desktopItem.getIcon()
             self.appCategories = desktopItem.getCategories()
-            self.appGnomeDocPath = desktopItem.get( "X-MATE-DocPath" ) or ""
+            self.appMateDocPath = desktopItem.get( "X-MATE-DocPath" ) or ""
             self.useTerminal = desktopItem.getTerminal()
             self.appPath = desktopItem.getPath()
 
-            if not self.appGnomeDocPath:
+            if not self.appMateDocPath:
                 self.appKdeDocPath      = desktopItem.getDocPath() or ""
 
             self.appName            = self.appName.strip()
@@ -332,9 +331,6 @@ class ApplicationLauncher( easyButton ):
     def onFocusOut( self, widget, event ):
         self.set_relief( Gtk.ReliefStyle.NONE )
 
-    def onEnterNotify( self, widget, event ):
-        self.grab_focus()
-
     def setupLabels( self ):
         self.addLabel( self.appName )
             
@@ -413,7 +409,7 @@ class ApplicationLauncher( easyButton ):
 
         shutil.copyfile( self.desktopFile, self.startupFilePath )
 
-        # Remove %u, etc. from Exec entry, because gnome will not replace them when it starts the app
+        # Remove %u, etc. from Exec entry, because MATE will not replace them when it starts the app
         item = matedesktop.item_new_from_uri( self.startupFilePath, matedesktop.LOAD_ONLY_IF_EXISTS )
         if item:
             r = re.compile("%[A-Za-z]");
